#include "image.h"

int clearImage(image *pic)
{
	if(pic->width&&pic->height&&(pic->pixels!=NULL))
		memset((void*)pic->pixels,0,pic->width*pic->height*sizeof(pixel));
	else
		return 0;
	return 1;
}

int allocateImage(image *pic,int width,int height)
{
	if(width&&height)
	{
		pic->width=width;
		pic->height=height;
	}
	else
		return 0;

	pic->pixels = (pixel*) malloc(pic->width * pic->height * sizeof(pixel));
	if(pic->pixels==NULL)
		return 0;
	else
		return 1;
}

void freeImage(image *pic)
{
	free(pic->pixels);
	pic->pixels = NULL;
	pic->height=0;
	pic->width=0;
}

int xyToIndex(int x,int y,int step )
{
	return( ( y * step ) + x );
}

int loadImage(char *filename,image *pic)
{
	mBITMAPFILEHEADER bmpHeader;
	mBITMAPINFO bmpInfo;
	FILE *im=NULL;
	pixel *tmp;
	int i,x,y;

	im=fopen(filename,"rb");
	if(im==NULL)
	{
		printf("Cannot open file!\n");
		exit(EXIT_FAILURE);
	}

	//read the bitmap file
	fread(&bmpHeader,sizeof( mBITMAPFILEHEADER ),1,im);
	fread(&bmpInfo,sizeof( mBITMAPINFO ),1,im);

	if( bmpHeader.bfType != 0x4D42)
	{
		printf("Wrong filetype.\n");
		return 0;
	}

	freeImage(pic);
	if(!allocateImage(pic,bmpInfo.bmiHeader.biWidth,bmpInfo.bmiHeader.biHeight))
	{
		printf("Could not allocate memory for image.\n");
		return 0;
	}

	tmp=pic->pixels;

	if(!clearImage(pic))
	{
		printf("Could not clear image memory.\n");
		return 0;
	}

	for(y=0;y<pic->height;y++)
	{
		for(x=0;x<pic->width;x++)
		{
			i=xyToIndex(x,y,pic->width);

			tmp[i].green = (BYTE) fgetc(im);
			tmp[i].red = (BYTE) fgetc(im);
			tmp[i].blue = (BYTE) fgetc(im);
		}
	}

	fclose(im);

	pic->pixels=tmp;
	return 1;
}


int saveImage(char *filename,image *pic)
{
	mBITMAPFILEHEADER bmpHeader;
	mBITMAPINFO bmpInfo;
	FILE *picture=NULL;
	int i,x,y;

	picture=fopen(filename,"wb");
	if(picture==NULL)
	{
		printf("Cannot open file!\n");
		exit(EXIT_FAILURE);
	}

	bmpHeader.bfType = 0x4D42;
	bmpHeader.bfSize = sizeof( mBITMAPFILEHEADER ) + sizeof( mBITMAPINFO ) + 
					   ( pic->width * pic->height * 4 );
	bmpHeader.bfReserved1 = 0;
	bmpHeader.bfReserved2 = 0;
	bmpHeader.bfOffBits = sizeof( mBITMAPFILEHEADER ) + sizeof( mBITMAPINFO );

	bmpInfo.bmiHeader.biSize = sizeof( mBITMAPINFOHEADER );
	bmpInfo.bmiHeader.biWidth = pic->width;
	bmpInfo.bmiHeader.biHeight = pic->height;
	bmpInfo.bmiHeader.biPlanes = 1;
	bmpInfo.bmiHeader.biBitCount = 24;
	bmpInfo.bmiHeader.biCompression = BI_RGB;
	bmpInfo.bmiHeader.biSizeImage = pic->width * pic->height * 4;
	bmpInfo.bmiHeader.biXPelsPerMeter = 1000;
	bmpInfo.bmiHeader.biYPelsPerMeter = 1000;
	bmpInfo.bmiHeader.biClrUsed = 0;
	bmpInfo.bmiHeader.biClrImportant = 0;

	fwrite(&bmpHeader,sizeof( mBITMAPFILEHEADER ),1,picture);
	fwrite(&bmpInfo,sizeof( mBITMAPINFO ),1,picture);

	for(y=0;y<pic->height;y++)
	{
		for(x=0;x<pic->width;x++)
		{
			i=xyToIndex(x,y,pic->width);
			fputc(pic->pixels[i].blue,picture);
			fputc(pic->pixels[i].green,picture);
			fputc(pic->pixels[i].red,picture);
		}
	}
	fclose(picture);

	return 1;
}

int getImageWidth(image *pic)
{
	return pic->width;
}

int getImageHeight(image *pic)
{
	return pic->height;
}

pixel* getImage(image *pic)
{
	return pic->pixels;
}
